addpath(genpath(pwd));
clear all;
rng('default'); rng(1);

% define a Gaussian mixture model
d = 1; k = 5;
mu = 10*rand(k,d); 
mu = mu - mean(mu);
sigma = eye(d,d);
weights = ones(1,k)/k;
model = gmdistribution(mu,sigma, weights);
X = random(model,100);

% plot the density 
try
figure; x = linspace(-20,20,1000)'; 
plot(x, pdf(model,x));
xlabel('x'); ylabel('p(x)');
end


%% Test our method: Kernel stein discrepancy (KSD) + bootstrap 
%%%%%%Case 1: p = q (model2 = model)
model2=model;
% goodness of fit test using KSD and U-statistics 
nboot = 1000;
score_q = @(x)score_function_gmm(model2, x);  % Define score function
[ksdu, pvalue.ksdu] = KSD_U_testBoot(X, score_q, 'bandwidth', 'median', 'nboot', nboot); 
if pvalue.ksdu < 0.05, 
    fprintf('pvalue=%f, Reject H0:  X is not drawn from model2\n',pvalue.ksdu);
else
    fprintf('pvalue=%f, Cannot reject H0:  X may be drawn from model2\n', pvalue.ksdu);    
end

%%%%%%Case 2: p \neq q  
% Define a perturbed model:                   
noise = randn(size(model.mu));  mu2 = model.mu + noise*1; 
model2=gmdistribution(mu2, model.Sigma, model.PComponents);

% goodness of fit test using KSD and U-statistics 
score_q = @(x)score_function_gmm(model2, x);  % Define score function
[ksdu, pvalue.ksdu, bootsample, info_KSD_U] = KSD_U_testBoot(X, score_q, 'bandwidth', 'median', 'nboot', nboot); 
if pvalue.ksdu < 0.05, 
    fprintf('pvalue=%f, Reject H0:  X is not drawn from model2\n',pvalue.ksdu);
else
    fprintf('pvalue=%f, Cannot reject H0:  X may be drawn from model2\n', pvalue.ksdu);    
end

%% Method 2: KSD test with Linear estimator:
score_q = @(x)score_function_gmm(model2, x);         
[ksdlinear, pvalue.ksdlinear, uq,infolinear] = KSD_linear(X, score_q, 'bandwidth', -1);          
pvalue
%% Method 3: MMD two sample test with Monte Carlo 
xsimu = random(model2, 100);                    
params.sig=-1; params.shuff=nboot; params.bootForce=1;    
[testStat2,thresh2,params2, pvalue.mmd] = mmdTestBoot(X,xsimu,0.05,params);       
pvalue

%% Other classical goodness of fit tests
[hks,pvalue.ks,ks2statV]=kstest(X, 'CDF', [X(:), cdf(model2,X)]); % Kolmogorov-Smirnov goodness-of-fit 
[hchi2,pvalue.chi2,kchi22statV]=chi2gof(X', 'CDF',  @(x)cdf(model2,x')); % Chi-square test   
[hcm2,pvalue.cm2,kcm2statV]=cmtest(X, 'CDF', [X(:), cdf(model2,X)]); % Cramer-Von Mises goodness-of-fit test
                    
[pvalue.logp, logp.dlogpLR, MCSampleLR] = LR_simple_simple_MC(model2, model, X, nboot); % likelihood ratio test (simple vs. simple)
pvalue
