function [ksd,p,bootstrapSamples, info] = KSD_V_statistics(x, score_q, varargin)
% Estimate kernelized Stein discrpancy using V-statistics, with bootstrap
% testing of H0: ksd = 0 vs H1: ksd \neq0
%
% x: sample of size Num_Instance X Dimension
% score_q: function handle of the score function of q; take x as input and
% output a column vector: Num_Instance X Dimension

[kernel, h, nboot] = process_varargin(varargin, 'kernel', 'rbf', 'width', -1, 'nboot', 0);
 
if h==-1, 
    h = sqrt(0.5*median_distance(x));  %rbf_dot has factor two in kernel
elseif h ==-2
    h = ratio_median_heuristic(x, score_q);    
end



if isa(score_q, 'char'), 
    switch lower(score_q)
        case 'gaussian'
        score_q = @(x)(-x); % standard normal 
    end
end

Sqx = score_q(x);
n=size(x,1); dim = size(x,2);

switch lower(kernel)
    case 'rbf'    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        XY = x*x';
        x2= sum(x.^2, 2);
        X2e = repmat(x2, 1, n);

        H = (X2e + X2e' - 2*XY);
        Kxy = exp(-H/(2*h^2));
        sqxdy = -(Sqx*x' - repmat(sum((Sqx.*x),2),1,n))./h^2;
        dxsqy = sqxdy';
        dxdy = (-H/h^4 + dim/h^2);
        M = (Sqx*Sqx' + sqxdy + dxsqy + dxdy).*Kxy;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
    otherwise
        error('wrong kernel');
end

M2 = M;% -diag(diag(M));
ksd = sum(sum(M2))/(n^2);
%ksd = ( sum(sum(M)) - trace(M) )/(n*(n-1));

%% bootstrap: 
bootstrapSamples = zeros(nboot,1);
bootmethod = process_varargin(varargin, 'bootmethod', 'efron');

switch lower(bootmethod)
    case 'efron'
        for i = 1:nboot
            xdx = randi(n,n,1);    
            bootstrapSamples(i) = ( sum(sum(M2(xdx,xdx))) ) / ( n^2);        
        end
        p = mean(bootstrapSamples<=0);
                        
    case 'weighted'
        for i = 1:nboot
            wtsboot= mnrnd(n,ones(1,n)/n)/n; 
            bootstrapSamples(i) = (wtsboot-1/n)*M2*(wtsboot' - 1/n);
        end
        %CenteredSample = bootstrapSamples-mean(bootstrapSamples); % %
        %recentering???
        %p = mean(  CenteredSample >= ksd ); 
        p = mean(  bootstrapSamples >= ksd );         
                        
    otherwise
        error('wrong bootmethod');
end

info.bandwidth = h;
info.M = M;
info.nboot = nboot;


return;
end


%{
function  [Kxy, dxKxy, dyKxy, dxyKxy]=rbf_with_scores(x, y, sqx, sqy, h)
% x: size NumInstance X NumDimension


n=size(x,1); p = size(x,2);

XY = x*x';
x2= sum(x.^2, 2);
X2e = repmat(x2, 1, n);

H = (X2e + X2e' - 2*XY);
Kxy = exp(-H/(2*h^2));

sqxdy = Kxy.*(Sqx*x' - repmat(sum((Sqx.*x),2),1,n))./h^2;
dxsqy = -sqxdy';
dxyKxy = Kxy.*(-H/h^4 + p/h^2);

M = (Sqx*Sqx' + sqxdy + dxsqy + dxdy).*Kxy;


%{
n=size(x,1); p = size(x,2);

XY = x*x';
x2= sum(x.^2, 2);
X2e = repmat(x2, 1, n);
xe = repmat(x, 1, n);

H = (X2e + X2e' - 2*XY);
Kxy = exp(-H/(2*h^2));
dxKxy = Kxy.*(xe' - xe)/(h^2);
dyKxy = -dxKxy;
dxyKxy = Kxy.*(-H/h^4 + 1/h^2);
%}

return;
end
%}

